library ieee;
use ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;


entity i2c_tb is
end;

architecture tb of i2c_tb is

	COMPONENT i2c_master IS
	  GENERIC(
		input_clk : INTEGER := 50_000_000; --input clock speed from user logic in Hz
		bus_clk   : INTEGER := 400_000);   --speed the i2c bus (scl) will run at in Hz
	  PORT(
		clk       : IN     STD_LOGIC;                    --system clock
		reset_n   : IN     STD_LOGIC;                    --active low reset
		ena       : IN     STD_LOGIC;                    --latch in command
		addr      : IN     STD_LOGIC_VECTOR(6 DOWNTO 0); --address of target slave
		rw        : IN     STD_LOGIC;                    --'0' is write, '1' is read
		data_wr   : IN     STD_LOGIC_VECTOR(7 DOWNTO 0); --data to write to slave
		busy      : OUT    STD_LOGIC;                    --indicates transaction in progress
		data_rd   : OUT    STD_LOGIC_VECTOR(7 DOWNTO 0); --data read from slave
		ack_error : BUFFER STD_LOGIC;                    --flag if improper acknowledge from slave
		sda       : INOUT  STD_LOGIC;                    --serial data output of i2c bus
		scl       : INOUT  STD_LOGIC);                   --serial clock output of i2c bus
	END COMPONENT;


	COMPONENT i2c_slave IS
	  generic (
	    	SLAVE_ADDR : std_logic_vector(6 downto 0));
	  port (
	    	scl              : inout std_logic;
	    	sda              : inout std_logic;
	    	clk              : in    std_logic;
	    	rst              : in    std_logic;
	    	-- User interface
	    	read_req         : out   std_logic;
	    	data_to_master   : in    std_logic_vector(7 downto 0);
	    	data_valid       : out   std_logic;
	    	data_from_master : out   std_logic_vector(7 downto 0));
	END COMPONENT;

	signal clk, reset_n, ena, rw, busy, sda, scl, ack_error : std_logic:='0';
	signal addr : std_logic_vector(6 downto 0):="1101001";
	signal data_wr, data_rd : std_logic_vector(7 downto 0):="00000000";
	signal divisor : std_logic_vector(6 downto 0) :="0000000";
	signal i : integer :=0;
	signal count : std_logic_vector(6 downto 0);
	signal	red_req : std_logic;
	signal data_to_master, data_from_master : std_logic_vector(7 downto 0);

begin

	--50Mhz system clock generation
	process
	begin
	while true loop
			clk<='1';
			wait for 10ns;
			clk<='0';
			wait for 10ns;
		end loop;
	end process;

	--40KHz SCL clock generation
	process (clk)
	begin
		if(rising_edge(clk))then
			if(count<"0111111")then
				scl<='0';
			else
				scl<='1';
				if (count="1111111")then
					count<="0000000";
				end if;
			end if;
			count<=count+'1';
		end if;
	end process;red_req




	UUT : i2c_master
		GENERIC MAP (SLAVE_ADDR=>"1110010");
		PORT MAP(
			clk => clk,
			reset_n => reset_n,
			ena => ena,
			addr => addr,
			rw => rw,
			data_wr => data_wr,
			busy => busy,
			data_rd => data_rd,
			ack_error => ack_error,
			sda => sda,
			scl => scl);
	UUT2: i2c_slave
	  port map(
	    	scl=>scl,
	    	sda=>sda,
	    	clk=>clk,
	    	rst=>reset_n,
	    	-- User interface
	    	read_req=>red_req,
	    	data_to_master=>data_to_master,
	    	data_valid=>ack_error,       : out   std_logic;
	    	data_from_master=>data_from_master);;
	END COMPONENT;


	
	stimulus : process 	--Casos de prueba.
	begin
		wait for 50ns;
		ena<='1';
		rw<='0';
		data_wr<="00000011";
		wait for 1000ns;
		data_wr<="11111111";
		
		wait for 20000ns;
		data_wr<="01010101";
	
		wait for 20000ns;
		data_wr<="11111100";

		wait for 50ns;

		wait;
	end process stimulus;
end tb ;